@@ -1,11 +1,12 @@
-from parcels.field import Field, VectorField, SummedField, SummedVectorField
+from parcels.field import Field, VectorField, SummedField, NestedField
 from parcels.tools.loggers import logger
 import ast
 import cgen as c
-from collections import OrderedDict
+import collections
 import math
 import numpy as np
 import random
+from copy import copy
 
 
 class IntrinsicNode(ast.AST):
@@ -19,12 +20,20 @@ class FieldSetNode(IntrinsicNode):
         if isinstance(getattr(self.obj, attr), Field):
             return FieldNode(getattr(self.obj, attr),
                              ccode="%s->%s" % (self.ccode, attr))
-        elif isinstance(getattr(self.obj, attr), SummedVectorField):
-            return SummedVectorFieldNode(getattr(self.obj, attr),
-                                         ccode="%s->%s" % (self.ccode, attr))
+        elif isinstance(getattr(self.obj, attr), NestedField):
+            if isinstance(getattr(self.obj, attr)[0], VectorField):
+                return NestedVectorFieldNode(getattr(self.obj, attr),
+                                             ccode="%s->%s" % (self.ccode, attr))
+            else:
+                return NestedFieldNode(getattr(self.obj, attr),
+                                       ccode="%s->%s" % (self.ccode, attr))
         elif isinstance(getattr(self.obj, attr), SummedField) or isinstance(getattr(self.obj, attr), list):
-            return SummedFieldNode(getattr(self.obj, attr),
-                                   ccode="%s->%s" % (self.ccode, attr))
+            if isinstance(getattr(self.obj, attr)[0], VectorField):
+                return SummedVectorFieldNode(getattr(self.obj, attr),
+                                             ccode="%s->%s" % (self.ccode, attr))
+            else:
+                return SummedFieldNode(getattr(self.obj, attr),
+                                       ccode="%s->%s" % (self.ccode, attr))
         elif isinstance(getattr(self.obj, attr), VectorField):
             return VectorFieldNode(getattr(self.obj, attr),
                                    ccode="%s->%s" % (self.ccode, attr))
@@ -77,8 +86,34 @@ class SummedVectorFieldNode(IntrinsicNode):
 
 
 class SummedVectorFieldEvalNode(IntrinsicNode):
-    def __init__(self, field, args, var, var2, var3):
-        self.field = field
+    def __init__(self, fields, args, var, var2, var3):
+        self.fields = fields
+        self.args = args
+        self.var = var  # the variable in which the interpolated field is written
+        self.var2 = var2  # second variable for UV interpolation
+        self.var3 = var3  # third variable for UVW interpolation
+
+
+class NestedFieldNode(IntrinsicNode):
+    def __getitem__(self, attr):
+        return NestedFieldEvalNode(self.obj, attr)
+
+
+class NestedFieldEvalNode(IntrinsicNode):
+    def __init__(self, fields, args, var):
+        self.fields = fields
+        self.args = args
+        self.var = var  # the variable in which the interpolated field is written
+
+
+class NestedVectorFieldNode(IntrinsicNode):
+    def __getitem__(self, attr):
+        return NestedVectorFieldEvalNode(self.obj, attr)
+
+
+class NestedVectorFieldEvalNode(IntrinsicNode):
+    def __init__(self, fields, args, var, var2, var3):
+        self.fields = fields
         self.args = args
         self.var = var  # the variable in which the interpolated field is written
         self.var2 = var2  # second variable for UV interpolation
@@ -175,7 +210,7 @@ class IntrinsicTransformer(ast.NodeTransformer):
 
     def get_tmp(self):
         """Create a new temporary veriable name"""
-        tmp = "tmp%d" % self._tmp_counter
+        tmp = "parcels_tmpvar%d" % self._tmp_counter
         self._tmp_counter += 1
         self.tmp_vars += [tmp]
         return tmp
@@ -194,6 +229,8 @@ class IntrinsicTransformer(ast.NodeTransformer):
             node = RandomNode(math, ccode='')
         elif node.id == 'print':
             node = PrintNode()
+        elif 'parcels_tmpvar' in node.id:
+            raise NotImplementedError("Custom Kernels cannot contain string 'parcels_tmpvar'; please change your kernel")
         return node
 
     def visit_Attribute(self, node):
@@ -221,9 +258,9 @@ class IntrinsicTransformer(ast.NodeTransformer):
             # .. and return the name of the temporary that will be populated
             return ast.Name(id='+'.join(tmp))
         elif isinstance(node.value, SummedVectorFieldNode):
-            tmp = [self.get_tmp() for _ in node.value.obj.U]
-            tmp2 = [self.get_tmp() for _ in node.value.obj.U]
-            tmp3 = [self.get_tmp() if node.value.obj.W else None for _ in node.value.obj.U]
+            tmp = [self.get_tmp() for _ in range(len(node.value.obj))]
+            tmp2 = [self.get_tmp() for _ in range(len(node.value.obj))]
+            tmp3 = [self.get_tmp() if list.__getitem__(node.value.obj, 0).vector_type == '3D' else None for _ in range(len(node.value.obj))]
             # Insert placeholder node for field eval ...
             self.stmt_stack += [SummedVectorFieldEvalNode(node.value, node.slice, tmp, tmp2, tmp3)]
             # .. and return the name of the temporary that will be populated
@@ -240,7 +277,7 @@ class IntrinsicTransformer(ast.NodeTransformer):
         elif isinstance(node.value, VectorFieldNode):
             tmp = self.get_tmp()
             tmp2 = self.get_tmp()
-            tmp3 = self.get_tmp() if node.value.obj.W else None
+            tmp3 = self.get_tmp() if node.value.obj.vector_type == '3D' else None
             # Insert placeholder node for field eval ...
             self.stmt_stack += [VectorFieldEvalNode(node.value, node.slice, tmp, tmp2, tmp3)]
             # .. and return the name of the temporary that will be populated
@@ -248,6 +285,19 @@ class IntrinsicTransformer(ast.NodeTransformer):
                 return ast.Tuple([ast.Name(id=tmp), ast.Name(id=tmp2), ast.Name(id=tmp3)], ast.Load())
             else:
                 return ast.Tuple([ast.Name(id=tmp), ast.Name(id=tmp2)], ast.Load())
+        elif isinstance(node.value, NestedFieldNode):
+            tmp = self.get_tmp()
+            self.stmt_stack += [NestedFieldEvalNode(node.value, node.slice, tmp)]
+            return ast.Name(id=tmp)
+        elif isinstance(node.value, NestedVectorFieldNode):
+            tmp = self.get_tmp()
+            tmp2 = self.get_tmp()
+            tmp3 = self.get_tmp() if list.__getitem__(node.value.obj, 0).vector_type == '3D' else None
+            self.stmt_stack += [NestedVectorFieldEvalNode(node.value, node.slice, tmp, tmp2, tmp3)]
+            if tmp3:
+                return ast.Tuple([ast.Name(id=tmp), ast.Name(id=tmp2), ast.Name(id=tmp3)], ast.Load())
+            else:
+                return ast.Tuple([ast.Name(id=tmp), ast.Name(id=tmp2)], ast.Load())
         else:
             return node
 
@@ -313,9 +363,9 @@ class KernelGenerator(ast.NodeVisitor):
     def __init__(self, fieldset, ptype):
         self.fieldset = fieldset
         self.ptype = ptype
-        self.field_args = OrderedDict()
-        self.vector_field_args = OrderedDict()
-        self.const_args = OrderedDict()
+        self.field_args = collections.OrderedDict()
+        self.vector_field_args = collections.OrderedDict()
+        self.const_args = collections.OrderedDict()
 
     def generate(self, py_ast, funcvars):
         # Replace occurences of intrinsic objects in Python AST
@@ -333,18 +383,21 @@ class KernelGenerator(ast.NodeVisitor):
         # Make sure that repeated variables are not declared more than
         # once. If variables occur in multiple Kernels, give a warning
         used_vars = []
+        funcvars_copy = copy(funcvars)  # editing a list while looping over it is dangerous
         for kvar in funcvars:
             if kvar in used_vars:
-                logger.warning(kvar+" declared in multiple Kernels")
-                funcvars.remove(kvar)
+                if kvar not in ['particle', 'fieldset', 'time']:
+                    logger.warning(kvar+" declared in multiple Kernels")
+                funcvars_copy.remove(kvar)
             else:
                 used_vars.append(kvar)
+        funcvars = funcvars_copy
         for kvar in self.kernel_vars + self.array_vars:
             if kvar in funcvars:
                 funcvars.remove(kvar)
         self.ccode.body.insert(0, c.Value('ErrorCode', 'err'))
         if len(funcvars) > 0:
-            self.ccode.body.insert(0, c.Value("float", ", ".join(funcvars)))
+            self.ccode.body.insert(0, c.Value("type_coord", ", ".join(funcvars)))
         if len(transformer.tmp_vars) > 0:
             self.ccode.body.insert(0, c.Value("float", ", ".join(transformer.tmp_vars)))
 
@@ -359,23 +412,18 @@ class KernelGenerator(ast.NodeVisitor):
         # Create function declaration and argument list
         decl = c.Static(c.DeclSpecifier(c.Value("ErrorCode", node.name), spec='inline'))
         args = [c.Pointer(c.Value(self.ptype.name, "particle")),
-                c.Value("double", "time"), c.Value("float", "dt")]
-        for field_name, field in self.field_args.items():
-            args += [c.Pointer(c.Value("CField", "%s" % field_name))]
-        for field_name, field in self.vector_field_args.items():
-            fieldset = field.fieldset
-            Wname = field.W.name if field.W else 'not_defined'
-            for f in [field.U.name, field.V.name, Wname]:
+                c.Value("double", "time")]
+        for field in self.field_args.values():
+            args += [c.Pointer(c.Value("CField", "%s" % field.ccode_name))]
+        for field in self.vector_field_args.values():
+            for fcomponent in ['U', 'V', 'W']:
                 try:
-                    # Next line will break for example if field.U was created but not added to the fieldset
-                    getattr(fieldset, f)
-                    if f not in self.field_args:
-                        args += [c.Pointer(c.Value("CField", "%s" % f))]
+                    f = getattr(field, fcomponent)
+                    if f.ccode_name not in self.field_args:
+                        args += [c.Pointer(c.Value("CField", "%s" % f.ccode_name))]
+                        self.field_args[f.ccode_name] = f
                 except:
-                    if f != Wname:
-                        raise RuntimeError("Field %s needed by a VectorField but it does not exist" % f)
-                    else:
-                        pass
+                    pass  # field.W does not always exist
         for const, _ in self.const_args.items():
             args += [c.Value("float", const)]
 
@@ -424,7 +472,7 @@ class KernelGenerator(ast.NodeVisitor):
                     pointer_args = True
                     continue
                 if isinstance(a, FieldNode) or isinstance(a, VectorFieldNode):
-                    a.ccode = a.obj.name
+                    a.ccode = a.obj.ccode_name
                 elif isinstance(a, ParticleNode):
                     continue
                 elif pointer_args:
@@ -492,7 +540,7 @@ class KernelGenerator(ast.NodeVisitor):
             self.visit(b)
         # field evals are replaced by a tmp variable is added to the stack.
         # Here it means field evals passes from node.test to node.body. We take it out manually
-        fieldInTestCount = node.test.ccode.count('tmp')
+        fieldInTestCount = node.test.ccode.count('parcels_tmpvar')
         body0 = c.Block([b.ccode for b in node.body[:fieldInTestCount]])
         body = c.Block([b.ccode for b in node.body[fieldInTestCount:]])
         orelse = c.Block([b.ccode for b in node.orelse]) if len(node.orelse) > 0 else None
@@ -626,26 +674,31 @@ class KernelGenerator(ast.NodeVisitor):
 
     def visit_FieldNode(self, node):
         """Record intrinsic fields used in kernel"""
-        self.field_args[node.obj.name] = node.obj
+        self.field_args[node.obj.ccode_name] = node.obj
 
     def visit_SummedFieldNode(self, node):
         """Record intrinsic fields used in kernel"""
         for fld in node.obj:
-            self.field_args[fld.name] = fld
+            self.field_args[fld.ccode_name] = fld
+
+    def visit_NestedFieldNode(self, node):
+        """Record intrinsic fields used in kernel"""
+        for fld in node.obj:
+            self.field_args[fld.ccode_name] = fld
 
     def visit_VectorFieldNode(self, node):
         """Record intrinsic fields used in kernel"""
-        self.vector_field_args[node.obj.name] = node.obj
+        self.vector_field_args[node.obj.ccode_name] = node.obj
 
     def visit_SummedVectorFieldNode(self, node):
         """Record intrinsic fields used in kernel"""
-        for fld in node.obj.U:
-            self.field_args[fld.name] = fld
-        for fld in node.obj.V:
-            self.field_args[fld.name] = fld
-        if hasattr(node.obj, 'W') and node.obj.W:
-            for fld in node.obj.W:
-                self.field_args[fld.name] = fld
+        for fld in node.obj:
+            self.vector_field_args[fld.ccode_name] = fld
+
+    def visit_NestedVectorFieldNode(self, node):
+        """Record intrinsic fields used in kernel"""
+        for fld in node.obj:
+            self.vector_field_args[fld.ccode_name] = fld
 
     def visit_ConstNode(self, node):
         self.const_args[node.ccode] = node.obj
@@ -665,14 +718,14 @@ class KernelGenerator(ast.NodeVisitor):
         ccode_eval = node.field.obj.ccode_eval(node.var, node.var2, node.var3,
                                                node.field.obj.U, node.field.obj.V, node.field.obj.W,
                                                *node.args.ccode)
-        if node.field.obj.U.interp_method != 'cgrid_linear':
+        if node.field.obj.U.interp_method != 'cgrid_velocity':
             ccode_conv1 = node.field.obj.U.ccode_convert(*node.args.ccode)
             ccode_conv2 = node.field.obj.V.ccode_convert(*node.args.ccode)
             statements = [c.Statement("%s *= %s" % (node.var, ccode_conv1)),
                           c.Statement("%s *= %s" % (node.var2, ccode_conv2))]
         else:
             statements = []
-        if node.var3:
+        if node.field.obj.vector_type == '3D':
             ccode_conv3 = node.field.obj.W.ccode_convert(*node.args.ccode)
             statements.append(c.Statement("%s *= %s" % (node.var3, ccode_conv3)))
         conv_stat = c.Block(statements)
@@ -691,27 +744,65 @@ class KernelGenerator(ast.NodeVisitor):
         node.ccode = c.Block(cstat)
 
     def visit_SummedVectorFieldEvalNode(self, node):
-        self.visit(node.field)
+        self.visit(node.fields)
         self.visit(node.args)
         cstat = []
-        if node.field.obj.W:
-            Wlist = node.field.obj.W
-        else:
-            Wlist = [None] * len(node.field.obj.U)
-        for U, V, W, var, var2, var3 in zip(node.field.obj.U, node.field.obj.V, Wlist, node.var, node.var2, node.var3):
-            vfld = VectorField(node.field.obj.name, U, V, W)
-            ccode_eval = vfld.ccode_eval(var, var2, var3, U, V, W, *node.args.ccode)
-            ccode_conv1 = U.ccode_convert(*node.args.ccode)
-            ccode_conv2 = V.ccode_convert(*node.args.ccode)
-            statements = [c.Statement("%s *= %s" % (var, ccode_conv1)),
-                          c.Statement("%s *= %s" % (var2, ccode_conv2))]
-            if var3:
-                ccode_conv3 = W.ccode_convert(*node.args.ccode)
+        for fld, var, var2, var3 in zip(node.fields.obj, node.var, node.var2, node.var3):
+            ccode_eval = fld.ccode_eval(var, var2, var3,
+                                        fld.U, fld.V, fld.W,
+                                        *node.args.ccode)
+            if fld.U.interp_method != 'cgrid_velocity':
+                ccode_conv1 = fld.U.ccode_convert(*node.args.ccode)
+                ccode_conv2 = fld.V.ccode_convert(*node.args.ccode)
+                statements = [c.Statement("%s *= %s" % (var, ccode_conv1)),
+                              c.Statement("%s *= %s" % (var2, ccode_conv2))]
+            else:
+                statements = []
+            if fld.vector_type == '3D':
+                ccode_conv3 = fld.W.ccode_convert(*node.args.ccode)
                 statements.append(c.Statement("%s *= %s" % (var3, ccode_conv3)))
-            conv_stat = c.Block(statements)
-            cstat += [c.Assign("err", ccode_eval), conv_stat, c.Statement("CHECKERROR(err)")]
+            cstat += [c.Assign("err", ccode_eval), c.Block(statements)]
+        cstat += [c.Statement("CHECKERROR(err)")]
         node.ccode = c.Block(cstat)
 
+    def visit_NestedFieldEvalNode(self, node):
+        self.visit(node.fields)
+        self.visit(node.args)
+        cstat = []
+        for fld in node.fields.obj:
+            ccode_eval = fld.ccode_eval(node.var, *node.args.ccode)
+            ccode_conv = fld.ccode_convert(*node.args.ccode)
+            conv_stat = c.Statement("%s *= %s" % (node.var, ccode_conv))
+            cstat += [c.Assign("err", ccode_eval),
+                      conv_stat,
+                      c.If("err != ERROR_OUT_OF_BOUNDS ", c.Block([c.Statement("CHECKERROR(err)"), c.Statement("break")]))]
+        cstat += [c.Statement("CHECKERROR(err)"), c.Statement("break")]
+        node.ccode = c.While("1==1", c.Block(cstat))
+
+    def visit_NestedVectorFieldEvalNode(self, node):
+        self.visit(node.fields)
+        self.visit(node.args)
+        cstat = []
+        for fld in node.fields.obj:
+            ccode_eval = fld.ccode_eval(node.var, node.var2, node.var3,
+                                        fld.U, fld.V, fld.W,
+                                        *node.args.ccode)
+            if fld.U.interp_method != 'cgrid_velocity':
+                ccode_conv1 = fld.U.ccode_convert(*node.args.ccode)
+                ccode_conv2 = fld.V.ccode_convert(*node.args.ccode)
+                statements = [c.Statement("%s *= %s" % (node.var, ccode_conv1)),
+                              c.Statement("%s *= %s" % (node.var2, ccode_conv2))]
+            else:
+                statements = []
+            if fld.vector_type == '3D':
+                ccode_conv3 = fld.W.ccode_convert(*node.args.ccode)
+                statements.append(c.Statement("%s *= %s" % (node.var3, ccode_conv3)))
+            cstat += [c.Assign("err", ccode_eval),
+                      c.Block(statements),
+                      c.If("err != ERROR_OUT_OF_BOUNDS ", c.Block([c.Statement("CHECKERROR(err)"), c.Statement("break")]))]
+        cstat += [c.Statement("CHECKERROR(err)"), c.Statement("break")]
+        node.ccode = c.While("1==1", c.Block(cstat))
+
     def visit_Return(self, node):
         self.visit(node.value)
         node.ccode = c.Statement('return %s' % node.value.ccode)
@@ -805,17 +896,19 @@ class LoopGenerator(object):
             args += [c.Pointer(c.Value("CField", "%s" % field))]
         for const, _ in const_args.items():
             args += [c.Value("float", const)]
-        fargs_str = ", ".join(['particles[p].time', 'sign_dt * __dt'] + list(field_args.keys())
+        fargs_str = ", ".join(['particles[p].time'] + list(field_args.keys())
                               + list(const_args.keys()))
         # Inner loop nest for forward runs
         sign_dt = c.Assign("sign_dt", "dt > 0 ? 1 : -1")
         particle_backup = c.Statement("%s particle_backup" % self.ptype.name)
         sign_end_part = c.Assign("sign_end_part", "endtime - particles[p].time > 0 ? 1 : -1")
         dt_pos = c.Assign("__dt", "fmin(fabs(particles[p].dt), fabs(endtime - particles[p].time))")
+        pdt_eq_dt_pos = c.Assign("particles[p].dt", "__dt * sign_dt")
         dt_0_break = c.If("particles[p].dt == 0", c.Statement("break"))
         notstarted_continue = c.If("(sign_end_part != sign_dt) && (particles[p].dt != 0)",
                                    c.Statement("continue"))
         body = [c.Statement("set_particle_backup(&particle_backup, &(particles[p]))")]
+        body += [pdt_eq_dt_pos]
         body += [c.Assign("res", "%s(&(particles[p]), %s)" % (funcname, fargs_str))]
         body += [c.Assign("particles[p].state", "res")]  # Store return code on particle
         body += [c.If("res == SUCCESS", c.Block([c.Statement("particles[p].time += sign_dt * __dt"),
