@@ -10,6 +10,18 @@ extern "C" {
 
 #define CHECKERROR(res) do {if (res != SUCCESS) return res;} while (0)
 
+#ifdef DOUBLE_COORD_VARIABLES
+typedef double type_coord;
+#else
+typedef float type_coord;
+#endif
+
+typedef enum
+  {
+    LINEAR=0, NEAREST=1, CGRID_VELOCITY=2, CGRID_TRACER=3, BGRID_VELOCITY=4, BGRID_W_VELOCITY=5, BGRID_TRACER=6,
+  } InterpCode;
+
+
 typedef struct
 {
   int gtype;
@@ -20,6 +32,8 @@ typedef struct
 {
   int xdim, ydim, zdim, tdim, z4d;
   int sphere_mesh, zonal_periodic;
+  double tfull_min, tfull_max;
+  int* periods;
   float *lonlat_minmax;
   float *lon, *lat, *depth;
   double *time;
@@ -36,7 +50,7 @@ typedef enum
     RECTILINEAR_Z_GRID=0, RECTILINEAR_S_GRID=1, CURVILINEAR_Z_GRID=2, CURVILINEAR_S_GRID=3
   } GridCode;
 
-static inline ErrorCode search_indices_vertical_z(float z, int zdim, float *zvals, int *zi, double *zeta)
+static inline ErrorCode search_indices_vertical_z(type_coord z, int zdim, float *zvals, int *zi, double *zeta)
 {
   if (z < zvals[0] || z > zvals[zdim-1]) {return ERROR_OUT_OF_BOUNDS;}
   while (*zi < zdim-1 && z > zvals[*zi+1]) ++(*zi);
@@ -47,10 +61,14 @@ static inline ErrorCode search_indices_vertical_z(float z, int zdim, float *zval
   return SUCCESS;
 }
 
-static inline ErrorCode search_indices_vertical_s(float z, int xdim, int ydim, int zdim, float *zvals,
+static inline ErrorCode search_indices_vertical_s(type_coord z, int xdim, int ydim, int zdim, float *zvals,
                                     int xi, int yi, int *zi, double xsi, double eta, double *zeta,
-                                    int z4d, int ti, int tdim, double time, double t0, double t1)
+                                    int z4d, int ti, int tdim, double time, double t0, double t1, int interp_method)
 {
+  if (interp_method == BGRID_VELOCITY || interp_method == BGRID_W_VELOCITY || interp_method == BGRID_TRACER){
+    xsi = 1;
+    eta = 1;
+  }
   float zcol[zdim];
   int zii;
   if (z4d == 1){
@@ -118,9 +136,9 @@ static inline void reconnect_bnd_indices(int *xi, int *yi, int xdim, int ydim, i
 }
 
 
-static inline ErrorCode search_indices_rectilinear(float x, float y, float z, CStructuredGrid *grid, GridCode gcode,
+static inline ErrorCode search_indices_rectilinear(type_coord x, type_coord y, type_coord z, CStructuredGrid *grid, GridCode gcode,
                                                    int *xi, int *yi, int *zi, double *xsi, double *eta, double *zeta,
-                                                   int ti, double time, double t0, double t1)
+                                                   int ti, double time, double t0, double t1, int interp_method)
 {
   int xdim = grid->xdim;
   int ydim = grid->ydim;
@@ -193,7 +211,7 @@ static inline ErrorCode search_indices_rectilinear(float x, float y, float z, CS
       case RECTILINEAR_S_GRID:
         err = search_indices_vertical_s(z, xdim, ydim, zdim, zvals,
                                         *xi, *yi, zi, *xsi, *eta, zeta,
-                                        z4d, ti, tdim, time, t0, t1);
+                                        z4d, ti, tdim, time, t0, t1, interp_method);
         break;
       default:
         err = ERROR;
@@ -203,18 +221,20 @@ static inline ErrorCode search_indices_rectilinear(float x, float y, float z, CS
   else
     *zeta = 0;
 
-  if ( (*xsi < 0) || (*xsi > 1) ) return ERROR_OUT_OF_BOUNDS;
-  if ( (*eta < 0) || (*eta > 1) ) return ERROR_OUT_OF_BOUNDS;
-  if ( (*zeta < 0) || (*zeta > 1) ) return ERROR_OUT_OF_BOUNDS;
+  if ( (*xsi < 0) || (*xsi > 1) ) return ERROR;
+  if ( (*eta < 0) || (*eta > 1) ) return ERROR;
+  if ( (*zeta < 0) || (*zeta > 1) ) return ERROR;
 
   return SUCCESS;
 }
 
 
-static inline ErrorCode search_indices_curvilinear(float x, float y, float z, CStructuredGrid *grid, GridCode gcode,
+static inline ErrorCode search_indices_curvilinear(type_coord x, type_coord y, type_coord z, CStructuredGrid *grid, GridCode gcode,
                                                    int *xi, int *yi, int *zi, double *xsi, double *eta, double *zeta,
-                                                   int ti, double time, double t0, double t1)
+                                                   int ti, double time, double t0, double t1, int interp_method)
 {
+  int xi_old = *xi;
+  int yi_old = *yi;
   int xdim = grid->xdim;
   int ydim = grid->ydim;
   int zdim = grid->zdim;
@@ -244,7 +264,8 @@ static inline ErrorCode search_indices_curvilinear(float x, float y, float z, CS
 
   *xsi = *eta = -1;
   int maxIterSearch = 1e6, it = 0;
-  while ( (*xsi < 0) || (*xsi > 1) || (*eta < 0) || (*eta > 1) ){
+  double tol = 1e-10;
+  while ( (*xsi < -tol) || (*xsi > 1+tol) || (*eta < -tol) || (*eta > 1+tol) ){
     double xgrid_loc[4] = {xgrid[*yi][*xi], xgrid[*yi][*xi+1], xgrid[*yi+1][*xi+1], xgrid[*yi+1][*xi]};
     if (sphere_mesh){ //we are on the sphere
       int i4;
@@ -285,28 +306,39 @@ static inline ErrorCode search_indices_curvilinear(float x, float y, float z, CS
       return ERROR_OUT_OF_BOUNDS;
     if ( (*xsi > 1) && (*eta > 1) && (*xi == xdim-1) && (*yi == ydim-1) )
       return ERROR_OUT_OF_BOUNDS;
-    if (*xsi < 0) 
+    if (*xsi < -tol)
       (*xi)--;
-    if (*xsi > 1)
+    if (*xsi > 1+tol)
       (*xi)++;
-    if (*eta < 0)
+    if (*eta < -tol)
       (*yi)--;
-    if (*eta > 1)
+    if (*eta > 1+tol)
       (*yi)++;
     reconnect_bnd_indices(xi, yi, xdim, ydim, 0, sphere_mesh);
     it++;
     if ( it > maxIterSearch){
       printf("Correct cell not found for (%f, %f) after %d iterations\n", x, y, maxIterSearch);
+      printf("Debug info: old particle indices: (yi, xi) %d %d\n", yi_old, xi_old);
+      printf("            new particle indices: (yi, xi) %d %d\n", *yi, *xi);
+      printf("            Mesh 2d shape:  %d %d\n", ydim, xdim);
+      printf("            Relative particle position:  (xsi, eta) %1.16e %1.16e\n", *xsi, *eta);
       return ERROR_OUT_OF_BOUNDS;
     }
   }
   if ( (*xsi != *xsi) || (*eta != *eta) ){  // check if nan
-      printf("xsi and or eta are nan values\n");
+      printf("Correct cell not found for (%f, %f))\n", x, y);
+      printf("Debug info: old particle indices: (yi, xi) %d %d\n", yi_old, xi_old);
+      printf("            new particle indices: (yi, xi) %d %d\n", *yi, *xi);
+      printf("            Mesh 2d shape:  %d %d\n", ydim, xdim);
+      printf("            Relative particle position:  (xsi, eta) %1.16e %1.16e\n", *xsi, *eta);
       return ERROR_OUT_OF_BOUNDS;
   }
+  if (*xsi < 0) *xsi = 0;
+  if (*xsi > 1) *xsi = 1;
+  if (*eta < 0) *eta = 0;
+  if (*eta > 1) *eta = 1;
 
   ErrorCode err;
-
   if (zdim > 1){
     switch(gcode){
       case CURVILINEAR_Z_GRID:
@@ -315,7 +347,7 @@ static inline ErrorCode search_indices_curvilinear(float x, float y, float z, CS
       case CURVILINEAR_S_GRID:
         err = search_indices_vertical_s(z, xdim, ydim, zdim, zvals,
                                         *xi, *yi, zi, *xsi, *eta, zeta,
-                                        z4d, ti, tdim, time, t0, t1);
+                                        z4d, ti, tdim, time, t0, t1, interp_method);
         break;
       default:
         err = ERROR;
@@ -325,9 +357,9 @@ static inline ErrorCode search_indices_curvilinear(float x, float y, float z, CS
   else
     *zeta = 0;
 
-  if ( (*xsi < 0) || (*xsi > 1) ) return ERROR_OUT_OF_BOUNDS;
-  if ( (*eta < 0) || (*eta > 1) ) return ERROR_OUT_OF_BOUNDS;
-  if ( (*zeta < 0) || (*zeta > 1) ) return ERROR_OUT_OF_BOUNDS;
+  if ( (*xsi < 0) || (*xsi > 1) ) return ERROR;
+  if ( (*eta < 0) || (*eta > 1) ) return ERROR;
+  if ( (*zeta < 0) || (*zeta > 1) ) return ERROR;
 
   return SUCCESS;
 }
@@ -335,20 +367,20 @@ static inline ErrorCode search_indices_curvilinear(float x, float y, float z, CS
 /* Local linear search to update grid index
  * params ti, sizeT, time. t0, t1 are only used for 4D S grids
  * */
-static inline ErrorCode search_indices(float x, float y, float z, CStructuredGrid *grid,
+static inline ErrorCode search_indices(type_coord x, type_coord y, type_coord z, CStructuredGrid *grid,
                                        int *xi, int *yi, int *zi, double *xsi, double *eta, double *zeta,
-                                       GridCode gcode, int ti, double time, double t0, double t1)
+                                       GridCode gcode, int ti, double time, double t0, double t1, int interp_method)
 {
   switch(gcode){
     case RECTILINEAR_Z_GRID:
     case RECTILINEAR_S_GRID:
       return search_indices_rectilinear(x, y, z, grid, gcode, xi, yi, zi, xsi, eta, zeta,
-                                   ti, time, t0, t1);
+                                   ti, time, t0, t1, interp_method);
       break;
     case CURVILINEAR_Z_GRID:
     case CURVILINEAR_S_GRID:
       return search_indices_curvilinear(x, y, z, grid, gcode, xi, yi, zi, xsi, eta, zeta,
-                                   ti, time, t0, t1);
+                                   ti, time, t0, t1, interp_method);
       break;
     default:
       printf("Only RECTILINEAR_Z_GRID, RECTILINEAR_S_GRID, CURVILINEAR_Z_GRID and CURVILINEAR_S_GRID grids are currently implemented\n");
@@ -357,22 +389,22 @@ static inline ErrorCode search_indices(float x, float y, float z, CStructuredGri
 }
 
 /* Local linear search to update time index */
-static inline ErrorCode search_time_index(double *t, int size, double *tvals, int *ti, int time_periodic)
+static inline ErrorCode search_time_index(double *t, int size, double *tvals, int *ti, int time_periodic, double tfull_min, double tfull_max, int *periods)
 {
   if (*ti < 0)
     *ti = 0;
   if (time_periodic == 1){
     if (*t < tvals[0]){
       *ti = size-1;
-      int periods = floor( (*t-tvals[0])/(tvals[size-1]-tvals[0]));
-      *t -= periods * (tvals[size-1]-tvals[0]);
-      search_time_index(t, size, tvals, ti, time_periodic);
+      *periods = (int) floor( (*t-tfull_min)/(tfull_max-tfull_min));
+      *t -= *periods * (tfull_max-tfull_min);
+      search_time_index(t, size, tvals, ti, time_periodic, tfull_min, tfull_max, periods);
     }  
-    else if (*t > tvals[size-1]){
+    else if (*t >= tvals[size-1]){
       *ti = 0;
-      int periods = floor( (*t-tvals[0])/(tvals[size-1]-tvals[0]));
-      *t -= periods * (tvals[size-1]-tvals[0]);
-      search_time_index(t, size, tvals, ti, time_periodic);
+      *periods = (int) floor( (*t-tfull_min)/(tfull_max-tfull_min));
+      *t -= *periods * (tfull_max-tfull_min);
+      search_time_index(t, size, tvals, ti, time_periodic, tfull_min, tfull_max, periods);
     }  
   }          
   while (*ti < size-1 && *t >= tvals[*ti+1]) ++(*ti);
