@@ -3,6 +3,7 @@ import numpy as np
 import netCDF4
 from datetime import timedelta as delta
 from parcels.tools.loggers import logger
+from parcels.tools.error import ErrorCode
 from os import path
 try:
     from parcels._version import version as parcels_version
@@ -13,6 +14,22 @@ except:
 __all__ = ['ParticleFile']
 
 
+def _is_particle_started_yet(particle, time):
+    """We don't want to write a particle that is not started yet.
+    Particle will be written if:
+      * particle.time is equal to time argument of pfile.write()
+      * particle.time is before time (in case particle was deleted between previous export and current one)
+    """
+    return (particle.dt*particle.time <= particle.dt*time or np.isclose(particle.time, time))
+
+
+def _set_calendar(origin_calendar):
+    if origin_calendar == 'np_datetime64':
+        return 'standard'
+    else:
+        return origin_calendar
+
+
 class ParticleFile(object):
     """Initialise netCDF4.Dataset for trajectory output.
 
@@ -74,22 +91,27 @@ class ParticleFile(object):
             self.time.units = "seconds"
         else:
             self.time.units = "seconds since " + str(self.particleset.time_origin)
-            self.time.calendar = self.particleset.time_origin.calendar
+            self.time.calendar = _set_calendar(self.particleset.time_origin.calendar)
         self.time.axis = "T"
 
-        self.lat = self.dataset.createVariable("lat", "f4", coords, fill_value=np.nan, chunksizes=self.chunksizes)
+        if self.particleset.lonlatdepth_dtype is np.float64:
+            lonlatdepth_precision = "f8"
+        else:
+            lonlatdepth_precision = "f4"
+
+        self.lat = self.dataset.createVariable("lat", lonlatdepth_precision, coords, fill_value=np.nan, chunksizes=self.chunksizes)
         self.lat.long_name = ""
         self.lat.standard_name = "latitude"
         self.lat.units = "degrees_north"
         self.lat.axis = "Y"
 
-        self.lon = self.dataset.createVariable("lon", "f4", coords, fill_value=np.nan, chunksizes=self.chunksizes)
+        self.lon = self.dataset.createVariable("lon", lonlatdepth_precision, coords, fill_value=np.nan, chunksizes=self.chunksizes)
         self.lon.long_name = ""
         self.lon.standard_name = "longitude"
         self.lon.units = "degrees_east"
         self.lon.axis = "X"
 
-        self.z = self.dataset.createVariable("z", "f4", coords, fill_value=np.nan, chunksizes=self.chunksizes)
+        self.z = self.dataset.createVariable("z", lonlatdepth_precision, coords, fill_value=np.nan, chunksizes=self.chunksizes)
         self.z.long_name = ""
         self.z.standard_name = "depth"
         self.z.units = "m"
@@ -151,28 +173,31 @@ class ParticleFile(object):
            (self.write_ondelete is False or deleted_only is True):
             if pset.size > 0:
 
-                first_write = [p for p in pset if (p.fileid < 0 or len(self.idx) == 0) and p.dt*p.time <= p.dt*time]  # len(self.idx)==0 in case pset is written to new ParticleFile
+                first_write = [p for p in pset if (p.fileid < 0 or len(self.idx) == 0) and _is_particle_started_yet(p, time)]  # len(self.idx)==0 in case pset is written to new ParticleFile
                 for p in first_write:
-                    p.fileid = self.lasttraj
+                    p.fileid = self.lasttraj  # particle id in current file
                     self.lasttraj += 1
 
                 self.idx = np.append(self.idx, np.zeros(len(first_write)))
 
                 for p in pset:
-                    if p.dt*p.time <= p.dt*time:  # don't write particles if they haven't started yet
+                    if _is_particle_started_yet(p, time):
                         i = p.fileid
                         self.id[i, self.idx[i]] = p.id
-                        self.time[i, self.idx[i]] = time
+                        self.time[i, self.idx[i]] = p.time
                         self.lat[i, self.idx[i]] = p.lat
                         self.lon[i, self.idx[i]] = p.lon
                         self.z[i, self.idx[i]] = p.depth
                         for var in self.user_vars:
                             getattr(self, var)[i, self.idx[i]] = getattr(p, var)
+                        if p.state != ErrorCode.Delete and not np.allclose(p.time, time):
+                            logger.warning_once('time argument in pfile.write() is %g, but a particle has time %g.' % (time, p.time))
+
                 for p in first_write:
                     for var in self.user_vars_once:
                         getattr(self, var)[p.fileid] = getattr(p, var)
             else:
-                logger.warning("ParticleSet is empty on writing as array")
+                logger.warning("ParticleSet is empty on writing as array at time %g" % time)
 
             if not deleted_only:
                 self.idx += 1
