@@ -5,7 +5,11 @@ from argparse import ArgumentParser
 from parcels import Field
 from os import environ
 from parcels.plotting import create_parcelsfig_axis, plotfield, cartopy_colorbar
+import sys
 try:
+    if sys.platform == 'darwin' and sys.version_info[0] == 3:
+        import matplotlib
+        matplotlib.use("TkAgg")
     import matplotlib.animation as animation
     from matplotlib import rc
 except:
@@ -47,14 +51,14 @@ def plotTrajectoriesFile(filename, mode='2d', tracerfile=None, tracerfield='P',
         record = pfile.variables[recordedvar]
     pfile.close()
 
-    if tracerfile is not None and mode is not 'hist2d':
+    if tracerfile is not None and mode != 'hist2d':
         tracerfld = Field.from_netcdf(tracerfile, tracerfield, {'lon': tracerlon, 'lat': tracerlat})
         plt, fig, ax, cartopy = plotfield(tracerfld)
         if plt is None:
             return  # creating axes was not possible
         titlestr = ' and ' + tracerfield
     else:
-        spherical = False if mode is '3d' or mesh == 'flat' else True
+        spherical = False if mode == '3d' or mesh == 'flat' else True
         plt, fig, ax, cartopy = create_parcelsfig_axis(spherical=spherical)
         if plt is None:
             return  # creating axes was not possible
@@ -93,7 +97,10 @@ def plotTrajectoriesFile(filename, mode='2d', tracerfile=None, tracerfield='P',
         if isinstance(plottimes[0], (np.datetime64, np.timedelta64)):
             plottimes = plottimes[~np.isnat(plottimes)]
         else:
-            plottimes = plottimes[~np.isnan(plottimes)]
+            try:
+                plottimes = plottimes[~np.isnan(plottimes)]
+            except:
+                pass
         b = time == plottimes[0]
         if cartopy:
             scat = ax.scatter(lon[b], lat[b], s=20, color='k', transform=cartopy.crs.Geodetic())
