@@ -1,7 +1,7 @@
 from math import cos, pi
 import numpy as np
 import cftime
-import netcdftime
+import inspect
 from datetime import timedelta as delta
 
 __all__ = ['UnitConverter', 'Geographic', 'GeographicPolar', 'GeographicSquare',
@@ -9,6 +9,17 @@ __all__ = ['UnitConverter', 'Geographic', 'GeographicPolar', 'GeographicSquare',
            'TimeConverter']
 
 
+def _get_cftime_datetimes():
+    # Is there a more elegant way to parse these from cftime?
+    cftime_calendars = tuple(x[1].__name__ for x in inspect.getmembers(cftime._cftime, inspect.isclass))
+    cftime_datetime_names = [ca for ca in cftime_calendars if 'Datetime' in ca]
+    return cftime_datetime_names
+
+
+def _get_cftime_calendars():
+    return [getattr(cftime, cf_datetime)(1990, 1, 1).calendar for cf_datetime in _get_cftime_datetimes()]
+
+
 class TimeConverter(object):
     """ Converter class for dates with different calendars in FieldSets
 
@@ -19,10 +30,9 @@ class TimeConverter(object):
     def __init__(self, time_origin=0):
         self.time_origin = 0 if time_origin is None else time_origin
         if isinstance(time_origin, np.datetime64):
-            self.calendar = "standard"
-        elif isinstance(time_origin, (cftime._cftime.DatetimeNoLeap,
-                                      netcdftime._netcdftime.DatetimeNoLeap)):
-            self.calendar = "NOLEAP"
+            self.calendar = "np_datetime64"
+        elif isinstance(time_origin, cftime._cftime.datetime):
+            self.calendar = time_origin.calendar
         else:
             self.calendar = None
 
@@ -34,9 +44,9 @@ class TimeConverter(object):
         :return: time - self.time_origin
         """
         time = time.time_origin if isinstance(time, TimeConverter) else time
-        if self.calendar == 'standard':
+        if self.calendar == 'np_datetime64':
             return (time - self.time_origin) / np.timedelta64(1, 's')
-        elif self.calendar == 'NOLEAP':
+        elif self.calendar in _get_cftime_calendars():
             if isinstance(time, (list, np.ndarray)):
                 return np.array([(t - self.time_origin).total_seconds() for t in time])
             else:
@@ -53,12 +63,12 @@ class TimeConverter(object):
         :return: self.time_origin + time
         """
         time = time.time_origin if isinstance(time, TimeConverter) else time
-        if self.calendar == 'standard':
+        if self.calendar == 'np_datetime64':
             if isinstance(time, (list, np.ndarray)):
                 return [self.time_origin + np.timedelta64(int(t), 's') for t in time]
             else:
                 return self.time_origin + np.timedelta64(int(time), 's')
-        elif self.calendar == 'NOLEAP':
+        elif self.calendar in _get_cftime_calendars():
             return self.time_origin + delta(seconds=time)
         elif self.calendar is None:
             return self.time_origin + time
