@@ -2,7 +2,7 @@
 from enum import IntEnum
 
 
-__all__ = ['ErrorCode', 'FieldSamplingError', 'TimeExtrapolationError',
+__all__ = ['ErrorCode', 'FieldSamplingError', 'FieldOutOfBoundError', 'TimeExtrapolationError',
            'KernelError', 'OutOfBoundsError', 'recovery_map']
 
 
@@ -29,6 +29,20 @@ class FieldSamplingError(RuntimeError):
         super(FieldSamplingError, self).__init__(message)
 
 
+class FieldOutOfBoundError(RuntimeError):
+    """Utility error class to propagate out-of-bound field sampling in Scipy mode"""
+
+    def __init__(self, x, y, z, field=None):
+        self.field = field
+        self.x = x
+        self.y = y
+        self.z = z
+        message = "%s sampled out-of-bound, at (%f, %f, %f)" % (
+            field.name if field else "Field", self.x, self.y, self.z
+        )
+        super(FieldOutOfBoundError, self).__init__(message)
+
+
 class TimeExtrapolationError(RuntimeError):
     """Utility error class to propagate erroneous time extrapolation sampling in Scipy mode"""
 
@@ -63,7 +77,7 @@ def parse_particletime(time, fieldset):
     return time
 
 
-def recovery_kernel_error(particle, fieldset, time, dt):
+def recovery_kernel_error(particle, fieldset, time):
     """Default error kernel that throws exception"""
     msg = "Error: %s" % particle.exception if particle.exception else None
     raise KernelError(particle, fieldset=fieldset, msg=msg)
@@ -95,7 +109,7 @@ class OutOfTimeError(KernelError):
         super(OutOfTimeError, self).__init__(particle, fieldset=fieldset, msg=message)
 
 
-def recovery_kernel_out_of_bounds(particle, fieldset, time, dt):
+def recovery_kernel_out_of_bounds(particle, fieldset, time):
     """Default sampling error kernel that throws OutOfBoundsError"""
     if particle.exception is None:
         # TODO: JIT does not yet provide the context that created
@@ -106,7 +120,7 @@ def recovery_kernel_out_of_bounds(particle, fieldset, time, dt):
         raise OutOfBoundsError(particle, fieldset, error.x, error.y, error.z)
 
 
-def recovery_kernel_time_extrapolation(particle, fieldset, time, dt):
+def recovery_kernel_time_extrapolation(particle, fieldset, time):
     """Default sampling error kernel that throws OutOfTimeError"""
     raise OutOfTimeError(particle, fieldset)
 
